<?
namespace Avos\Redis;

//
// return sites redis prefix optionally with /key appended
//

function getPrefix($key='')
{
	global $_AVOS;

	if (!isset($_AVOS['config']['redis']['prefix'])) {
		return $key;
	}

	if (!empty($key)) {
		$key = '/' . $key;
	}

	return $_AVOS['config']['redis']['prefix'] . $key;
}

function handle()
{
	global $_AVOS;
	return $_AVOS['redis'];
}

//
// return all rank info
//

function getRanks($webcamsId, $normalised=0, $months=0)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	if ($months > 0) {
		$keys = array();
		$yymm = \Avos\Utils\yearsMonthsKeys($months);

		foreach ($yymm as $date) {
			$keys[] = "$webcamsId/r/$date";
		}
	}

	sort($keys);
	$ranks = array();

	foreach ($keys as $k) {
		$tmp = $redis->lRange($k, 0, -1);

		if (count($tmp) == 0) {
			$ranks = \Avos\SQLite\get($k);
		}

		$ranks = array_merge($ranks, $tmp);
	}

	$max = array('r' => 0, 'g' => 0, 'v' => 0, 'f' => 0);
	$tmax = 0;

	// find max for each attribute and of all
	foreach ($ranks as $r) {
		foreach (array('r', 'g', 'v', 'f') as $f) {
			if (!isset($r[$f])) {
				$r[$f] = 0;
			}

			if ($r[$f] > $max[$f]) {
				$max[$f] = $r[$f];
			}
			if ($r[$f] > $tmax) {
				$tmax = $r[$f];
			}
		}
	}

	// build return lists
	$ret = array(
				'epochTime' => array(),
				'dateStr' => array(),
				'rank' => array(), 
				'genderRank' => array(), 
				'viewers' => array(), 
				'followers' => array()
			);

	$map = array('r' => 'rank', 'g' => 'genderRank', 'v' => 'viewers', 'f' => 'followers');

	// normalise the numbers from 1 thru 100000
	foreach ($ranks as $r) {
		foreach (array('r', 'g', 'v', 'f') as $f) {

			// should fix old data where we didnt collect viewers+followers
			if (!isset($r[$f])) {
				$r[$f] = 0;
			}

			// in case we don't have followers
			if ($max[$f] == 0) {
				$max[$f] = 1;
			}

			if ($normalised > 0) {
				$ret[$map[$f]][] = $r[$f] * $normalised / $max[$f];
			}
			else {
				$ret[$map[$f]][] = $r[$f];
			}
		}

		$ret['epochTime'][] = $r['e'] * 60 + EPOCHSTART;
		$ret['dateStr'][] = date('y M y g:i a', $r['e'] * 60 + EPOCHSTART);
	}

	// reverse
	foreach ($ret as $k => $v) {
		$ret[$k] = array_reverse($v);
	}

	return $ret;
}

//
// get a model attribute
// * if isList then append $k to the key and use a list
//   else set a value
//

function getModel($webcamsId, $k='', $isList=false)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$key = "$webcamsId/p";

	if ($isList) {
		$key .= "/$k";
		$ret = $redis->lRange($key, 0, -1);
		#$ret = \Avos\SQLite\get($key);
	}
	elseif (empty($k)) {
		$ret = $redis->hGetAll($key);
		#$ret = \Avos\SQLite\get($key);
	}
	else {
		$ret = $redis->hGet($key, $k);
		#$ret = \Avos\SQLite\get($key);
	}

	return $ret;
}

//
// set a model attribute
//

function setModel($webcamsId, $k, $v, $isList=false)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$key = "$webcamsId/p";

	try {
		if ($isList) {
			$key = "$webcamsId/$k";
			#echo "appending [$key] with [$v]\n";
			$redis->rPush($key, $v);
		}
		else {
			#echo "setting [$key.$k] to [$v]\n";
			$redis->hSet($key, $k, $v);
		}
	}
	catch (Exception $e) {
		echo "Avos\Redis\setModel unable to save data\n";
	}
}

//
// either call with just webcamsId or webcamsId + year + month
//
// redis stores elements as e, s, t
// return as epochTime, status, statusTime
//

function getHistoryMonth($webcamsId, $year='', $month='')
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// need leading zero
	$month = sprintf("%02d", $month);
	$key = "$webcamsId/h/$year$month";

	$tmp = $redis->lRange($key, 0, -1);
	if (count($tmp) == 0) {
		$tmp = \Avos\SQLite\get($key);
	}

	$ret = array();

	foreach ($tmp as $v) {
		$ret[] = array(
					'webcamsId' => $webcamsId,
					'epochTime' => $v['e']*60 + EPOCHSTART, 
					'status' => $v['s'], 
					'statusTime' => $v['t']
				);
	}

	return $ret;
}

//
// if count=0 then just return the all else return at least count amounts
// if reverse=true then return from newest to oldest
//

function getHistory($webcamsId, $count=0, $reverse=false, $months=2)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$keys = array();
	$ret = array();

	$year = date('y');
	$month = date('n');

	while ($months-- > 0) {
		$keys[] = "$webcamsId/h/$year" . sprintf("%02d", $month - $months);
	}

	//
	// sort the keys
	//

	if ($reverse == true) {
		rsort($keys);
	}
	else {
		sort($keys);
	}

	//
	// extract data for specified months
	//

	foreach ($keys as $key) {

		// check sqlite if the key doesnt exist in redis
		$tmp = $redis->lRange($key, 0, -1);
		if (count($tmp) == 0) {
			$tmp = \Avos\SQLite\get($key);
		}

		$tmp2 = array();

		foreach ($tmp as $v) {
			$tmp2[] = array(
						'webcamsId' => $webcamsId,
						'epochTime' => $v['e'], 
						'status' => $v['s'], 
						'statusTime' => $v['t']
					);
		}

		// if we need in reverse order then 
		$tmp2 = array_reverse($tmp2);

		// append to return array
		foreach ($tmp2 as $t) {
			$ret[] = $t;
		}

		// check to see if we have enough
		if ($count > 0 && count($ret) >= $count) {
			return $ret;
		}
	}

	// return all we have
	return $ret;
}

//
// expand keys from short internal format
//

function expandStat($webcamsId, $year, $month, $day, $v)
{
	return array(
		'webcamsId' => $webcamsId,
		'year' => $year,
		'month' => $month,
		'day' => $day,

		'awaySessions' => $v['as'], 
		'awayTime' => $v['at'], 
		'awayTimeMax' => $v['atm'],

		'freechatSessions' => $v['fs'], 
		'freechatTime' => $v['ft'], 
		'freechatTimeMax' => $v['ftm'],

		'groupSessions' => $v['gs'], 
		'groupTime' => $v['gt'], 
		'groupTimeMax' => $v['gtm'],

		'privateSessions' => $v['ps'], 
		'privateTime' => $v['pt'], 
		'privateTimeMax' => $v['ptm']
	);
}

//
// set a daily or monthly stats entry
//

function setStat($webcamsId, $mode="m", $year, $month, $day, $value)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$year = sprintf("%02d", $year);
	$month = sprintf("%02d", $month);

	if (!empty($day)) {
		$day = sprintf("%02d", $day);
	}

	if ($mode == 'm') {
		$key = "$webcamsId/m/$year$month";
	}
	else {
		$key = "$webcamsId/d/$year$month$day";
	}

	#if (DEBUG) echo "setting ($key) => "; print_r($value);
	$redis->set($key, $value);
}

//
// return only a single record based on the key info provided
//

function getStat($webcamsId, $mode="m", $year="", $month="", $day="", $expandNames=true)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// year
	if (!empty($year)) {
		$year = sprintf("%02d", (int)$year);
	}

	// month
	if (!empty($month)) {
		$month = sprintf("%02d", (int)$month);
	}

	// day
	if (!empty($day)) {
		$day = sprintf("%02d", (int)$day);
	}

	// create the key
	$key = "$webcamsId/$mode/$year$month";
	if ($mode == 'd') {
		$key .= $day;
	}

	// expand if we need to
	$ret = $redis->get($key);
	if ($ret == false) {
		$ret = \Avos\SQLite\get($key);
	}

	if ($expandNames == true) {
		$ret = expandStat($webcamsId, $year, $month, $day, $ret);
	}

	return $ret;
}

//
// mode: m=monthly, d=daily
//
// XXXX/m/YYMM     => single entry
// XXXX/m/YY       => multiple entries
// XXXX/d/YYMMDD   => single entry
// XXXX/d/YYMM     => multiple entries
// XXXX/d/YY       => multiple entries
//

function getStats($webcamsId, $mode="m", $year="*", $month="*", $day="*", $reverse=false, $expandNames=true)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	// year
	if (!empty($year) && $year != '*') {
		$year = sprintf("%02d", $year);
	}

	// month
	if (!empty($month) && $month != '*') {
		$month = sprintf("%02d", $month);
	}

	// day
	if ($mode == 'd' && !empty($day) && $day != '*') {
		$day = sprintf("%02d", $day);
	}

	// create key with 0 or 1 wildcard '*'
	$search = "$webcamsId/$mode/$year";

	if ($year != "*") {
		$search .= $month;

		if ($month != "*" && $mode == 'd') {
			$search .= $day;
		}
	}

	//
	// search for specified years/months
	//

	$keys = array();

	if ((int)$month >= 0) {
		if ($mode == 'd') {
			for ($i=1; $i<31; $i++) {
				$keys[] = sprintf("$webcamsId/d/%s%s%02d", $year, $month, $i);
			}
		}
		else {
			//$keys = $redis->keys($search);
			echo "Avos\Redis\getModel() will not run search\n";
			return array();
		}
	}

	//
	// search back required number of months
	// e.g: $month = -10 then search back 10 months
	//

	else {
		$days = $mode == 'd' ? true : false;
		$dates = \Avos\Utils\yearsMonthsKeys(0 - $month, $days);

		foreach ($dates as $d) {
			$keys[] = sprintf("$webcamsId/$mode/$d");
		}
	}

	//
	// sort the keys
	//

	#echo "search=$search\n";exit;
	$ret = array();

	if ($reverse == true) {
		rsort($keys);
	}
	else {
		sort($keys);
	}

	//
	// extract the stats
	//

	foreach ($keys as $key) {

		$v = $redis->get($key);
		if (empty($v) || count($v) == 0) {
			$v = \Avos\SQLite\get($key);
		}

		if (is_array($v) && count($v) > 0) {
			preg_match("!/(\d\d)(\d\d)(\d\d)?$!", $key, $c);
			$year = (int)$c[1];
			$month = (int)$c[2];
			$day = isset($c[3]) ? (int)$c[3] : '';

			if ($expandNames == true) {
				$ret[] = expandStat($webcamsId, $year, $month, $day, $v);
			}
			else {
				$ret[] = $v;
			}
		}
	}

	return $ret;
}


